# -*- coding: utf-8 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

NOMPROG = u"Géophar"
NOMPROG2 = 'Geophar' # sans accent

# Numéro de version (et date de sa sortie)
version = '16.08.4'
date_version = (2016, 11, 6)
git = 'v16.08.3-2-gd654f7a'