# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'wxgeometrie/GUI/reperage.ui'
#
# Created: Thu Jul 10 18:02:21 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_DialogReperage(object):
    def setupUi(self, DialogReperage):
        DialogReperage.setObjectName(_fromUtf8("DialogReperage"))
        DialogReperage.resize(444, 493)
        self.verticalLayout = QtGui.QVBoxLayout(DialogReperage)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBoxRepere = QtGui.QGroupBox(DialogReperage)
        self.groupBoxRepere.setObjectName(_fromUtf8("groupBoxRepere"))
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.groupBoxRepere)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label = QtGui.QLabel(self.groupBoxRepere)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.repere_O = QtGui.QLineEdit(self.groupBoxRepere)
        self.repere_O.setObjectName(_fromUtf8("repere_O"))
        self.horizontalLayout.addWidget(self.repere_O)
        self.label_2 = QtGui.QLabel(self.groupBoxRepere)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout.addWidget(self.label_2)
        self.repere_I = QtGui.QLineEdit(self.groupBoxRepere)
        self.repere_I.setObjectName(_fromUtf8("repere_I"))
        self.horizontalLayout.addWidget(self.repere_I)
        self.label_3 = QtGui.QLabel(self.groupBoxRepere)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout.addWidget(self.label_3)
        self.repere_J = QtGui.QLineEdit(self.groupBoxRepere)
        self.repere_J.setObjectName(_fromUtf8("repere_J"))
        self.horizontalLayout.addWidget(self.repere_J)
        self.label_4 = QtGui.QLabel(self.groupBoxRepere)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.horizontalLayout.addWidget(self.label_4)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.verticalLayout.addWidget(self.groupBoxRepere)
        self.groupBoxGraduations = QtGui.QGroupBox(DialogReperage)
        self.groupBoxGraduations.setObjectName(_fromUtf8("groupBoxGraduations"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.groupBoxGraduations)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label_5 = QtGui.QLabel(self.groupBoxGraduations)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.horizontalLayout_2.addWidget(self.label_5)
        self.gradu_x = QtGui.QLineEdit(self.groupBoxGraduations)
        self.gradu_x.setObjectName(_fromUtf8("gradu_x"))
        self.horizontalLayout_2.addWidget(self.gradu_x)
        self.label_7 = QtGui.QLabel(self.groupBoxGraduations)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.horizontalLayout_2.addWidget(self.label_7)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.label_6 = QtGui.QLabel(self.groupBoxGraduations)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.horizontalLayout_3.addWidget(self.label_6)
        self.gradu_y = QtGui.QLineEdit(self.groupBoxGraduations)
        self.gradu_y.setObjectName(_fromUtf8("gradu_y"))
        self.horizontalLayout_3.addWidget(self.gradu_y)
        self.label_8 = QtGui.QLabel(self.groupBoxGraduations)
        self.label_8.setObjectName(_fromUtf8("label_8"))
        self.horizontalLayout_3.addWidget(self.label_8)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.verticalLayout.addWidget(self.groupBoxGraduations)
        self.groupBoxEchelle = QtGui.QGroupBox(DialogReperage)
        self.groupBoxEchelle.setObjectName(_fromUtf8("groupBoxEchelle"))
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.groupBoxEchelle)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.label_9 = QtGui.QLabel(self.groupBoxEchelle)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.horizontalLayout_4.addWidget(self.label_9)
        self.echelle_x = QtGui.QLineEdit(self.groupBoxEchelle)
        self.echelle_x.setObjectName(_fromUtf8("echelle_x"))
        self.horizontalLayout_4.addWidget(self.echelle_x)
        self.label_12 = QtGui.QLabel(self.groupBoxEchelle)
        self.label_12.setObjectName(_fromUtf8("label_12"))
        self.horizontalLayout_4.addWidget(self.label_12)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.label_13 = QtGui.QLabel(self.groupBoxEchelle)
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.horizontalLayout_5.addWidget(self.label_13)
        self.echelle_y = QtGui.QLineEdit(self.groupBoxEchelle)
        self.echelle_y.setObjectName(_fromUtf8("echelle_y"))
        self.horizontalLayout_5.addWidget(self.echelle_y)
        self.label_10 = QtGui.QLabel(self.groupBoxEchelle)
        self.label_10.setObjectName(_fromUtf8("label_10"))
        self.horizontalLayout_5.addWidget(self.label_10)
        spacerItem4 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.verticalLayout_5.addLayout(self.horizontalLayout_5)
        self.rapport = QtGui.QCheckBox(self.groupBoxEchelle)
        self.rapport.setObjectName(_fromUtf8("rapport"))
        self.verticalLayout_5.addWidget(self.rapport)
        self.verticalLayout.addWidget(self.groupBoxEchelle)
        self.groupBoxResolution = QtGui.QGroupBox(DialogReperage)
        self.groupBoxResolution.setObjectName(_fromUtf8("groupBoxResolution"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.groupBoxResolution)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.label_15 = QtGui.QLabel(self.groupBoxResolution)
        self.label_15.setObjectName(_fromUtf8("label_15"))
        self.verticalLayout_2.addWidget(self.label_15)
        self.horizontalLayout_7 = QtGui.QHBoxLayout()
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.label_14 = QtGui.QLabel(self.groupBoxResolution)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.horizontalLayout_7.addWidget(self.label_14)
        self.resolution = QtGui.QLineEdit(self.groupBoxResolution)
        self.resolution.setObjectName(_fromUtf8("resolution"))
        self.horizontalLayout_7.addWidget(self.resolution)
        self.label_11 = QtGui.QLabel(self.groupBoxResolution)
        self.label_11.setObjectName(_fromUtf8("label_11"))
        self.horizontalLayout_7.addWidget(self.label_11)
        spacerItem5 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem5)
        self.calibrage = QtGui.QPushButton(self.groupBoxResolution)
        self.calibrage.setObjectName(_fromUtf8("calibrage"))
        self.horizontalLayout_7.addWidget(self.calibrage)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.verticalLayout.addWidget(self.groupBoxResolution)
        self.line = QtGui.QFrame(DialogReperage)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.verticalLayout.addWidget(self.line)
        self.boutons = QtGui.QDialogButtonBox(DialogReperage)
        self.boutons.setOrientation(QtCore.Qt.Horizontal)
        self.boutons.setStandardButtons(QtGui.QDialogButtonBox.Apply|QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok|QtGui.QDialogButtonBox.Reset)
        self.boutons.setObjectName(_fromUtf8("boutons"))
        self.verticalLayout.addWidget(self.boutons)

        self.retranslateUi(DialogReperage)
        QtCore.QObject.connect(self.boutons, QtCore.SIGNAL(_fromUtf8("accepted()")), DialogReperage.accept)
        QtCore.QObject.connect(self.boutons, QtCore.SIGNAL(_fromUtf8("rejected()")), DialogReperage.close)
        QtCore.QMetaObject.connectSlotsByName(DialogReperage)

    def retranslateUi(self, DialogReperage):
        DialogReperage.setWindowTitle(_translate("DialogReperage", "Personnaliser le repère", None))
        self.groupBoxRepere.setTitle(_translate("DialogReperage", "Repère", None))
        self.label.setText(_translate("DialogReperage", "Entrer le repère : (", None))
        self.repere_O.setToolTip(_translate("DialogReperage", "<html><head/><body><p>Légende à afficher pour l\'origine du repère.</p></body></html>", None))
        self.label_2.setText(_translate("DialogReperage", ";", None))
        self.repere_I.setToolTip(_translate("DialogReperage", "<html><head/><body><p>Légende à afficher pour le 2e point du repère (abscisse).</p><p><span style=\" font-weight:600;\">Nota : </span>si une lettre en minuscule est rentrée, elle sera comprise comme un vecteur.</p></body></html>", None))
        self.label_3.setText(_translate("DialogReperage", ";", None))
        self.repere_J.setToolTip(_translate("DialogReperage", "<html><head/><body><p>Légende à afficher pour le 3e point du repère (ordonnée). </p><p><span style=\" font-weight:600;\">Nota : </span>si une lettre en minuscule est rentrée, elle sera comprise comme un vecteur.</p></body></html>", None))
        self.label_4.setText(_translate("DialogReperage", ")", None))
        self.groupBoxGraduations.setTitle(_translate("DialogReperage", "Graduations", None))
        self.label_5.setText(_translate("DialogReperage", "Graduer en abscisse toutes les ", None))
        self.label_7.setText(_translate("DialogReperage", "unités", None))
        self.label_6.setText(_translate("DialogReperage", "Graduer en ordonnée toutes les ", None))
        self.label_8.setText(_translate("DialogReperage", "unités", None))
        self.groupBoxEchelle.setTitle(_translate("DialogReperage", "Échelle", None))
        self.label_9.setText(_translate("DialogReperage", "Unité en abscisse :", None))
        self.label_12.setText(_translate("DialogReperage", "cm", None))
        self.label_13.setText(_translate("DialogReperage", "Unité en ordonnée :", None))
        self.label_10.setText(_translate("DialogReperage", "cm", None))
        self.rapport.setToolTip(_translate("DialogReperage", "<html><head/><body><p>Cochez cette case pour conserver un rapport abscisse/ordonnée constant lorsque vous zoomez ou redimensionnez la fenêtre.</p></body></html>", None))
        self.rapport.setText(_translate("DialogReperage", "Conserver ce rapport abscisse/ordonnée lors d\'un zoom", None))
        self.groupBoxResolution.setTitle(_translate("DialogReperage", "Résolution", None))
        self.label_15.setText(_translate("DialogReperage", "<html><head/><body><p>Ajuster ce paramètre en fonction de votre écran.<br/><span style=\" font-style:italic;\">Ceci permet d\'affiner l\'affichage pour qu\'un centimètre <br/>mesure vraiment un centimètre sur votre écran !</span></p></body></html>", None))
        self.label_14.setText(_translate("DialogReperage", "Résolution", None))
        self.resolution.setToolTip(_translate("DialogReperage", "<html><head/><body><p>Entrez ici la résolution de votre moniteur en dpi. Exemple : 96.</p></body></html>", None))
        self.label_11.setText(_translate("DialogReperage", "dpi", None))
        self.calibrage.setToolTip(_translate("DialogReperage", "<html><head/><body><p>Afficher un objet à mesurer pour calibrer l\'écran.</p></body></html>", None))
        self.calibrage.setText(_translate("DialogReperage", "Outil de calibrage", None))

