# -*- coding: utf-8 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

afficher_axes = False
afficher_quadrillage = False
placement_probabilites = ['dessus', 'longe', 'decale'][1]
