# -*- coding: utf-8 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

from pytest import XFAIL

from tools.testlib import rand
from wxgeometrie.geolib import Champ, Bouton


@XFAIL
def test_Champ():
    raise NotImplementedError

@XFAIL
def test_Bouton():
    raise NotImplementedError
